import java.awt.*;
import javax.swing.*;
import java.util.Vector;
import java.io.*;

class Graphical {
	public static void main(String[] args) throws IOException {
		
		BufferedReader stdin = 
			new BufferedReader( new InputStreamReader( System.in ) );
			
		// Ask user for input
		System.out.println("Which dimensions do you want the board to have? (min 5 by 5)");
		System.out.println("Width: (in boardspaces)");
		String width = stdin.readLine();
		System.out.println("Height: (in boardspaces)");
		String height = stdin.readLine();
		
		// Set given Dimensions
		try {
		FinalHorse.setDim(Integer.parseInt(width),Integer.parseInt(height));
		}
		catch(Exception e) {
			System.out.println(e);
			System.exit(0);
		}
		
		// Ask user for input
		System.out.println("");
		System.out.println("How many of the "+ FinalHorse.dimx*FinalHorse.dimy+" locations do you want to be heuristicaly determined?");
		String precalc = stdin.readLine();
		Vector path = FinalHorse.findpath(new HorsePoint(0,FinalHorse.dimy-1,1), Integer.parseInt(precalc), true);
		
		int[][] b = new int[Integer.parseInt(width)][Integer.parseInt(height)];
		
		JFrame kframe = new JFrame("Knights Tour Search");
		Container pane = kframe.getContentPane();
		
		DrawingCanvas kcanvas = new DrawingCanvas(b, 30);
		pane.add(kcanvas);
		kframe.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		kframe.pack();
		kframe.show();
		kframe.toFront();
		
		animate(path,700,kcanvas);
		}

		private static void animate(Vector g,int pause, DrawingCanvas screen) {
		for(int i=0; i < g.size(); i++) {
		HorsePoint p = (HorsePoint) g.elementAt(i);
		screen.setCell(p.x, p.y,i+1);
		screen.repaint();
		try{
		Thread.sleep(pause);
		}
		catch(InterruptedException e){
		System.out.println("Sleep Interrupted");
		}
		}
		}
}

class DrawingCanvas extends JComponent {
private static final int FONTSIZE = 10;
private int board[][];
private Image horse;

private int xsize, ysize;
private int lastx=-1, lasty=-1;
private int cellsize;
	
	DrawingCanvas(int[][] board, int cellsize) {
		this.board = board;
		this.xsize = board.length * cellsize;
		this.ysize = board[0].length * cellsize;
		this.cellsize = cellsize;
		this.horse = Toolkit.getDefaultToolkit().getImage("knight.gif");
	}
	
	public void paint(Graphics g) {
		drawboard(g);
		Font f = new Font("Dialog",Font.PLAIN,FONTSIZE);
		FontMetrics fm = getFontMetrics(f);
		g.setFont(f);
		for(int i = 0; i < board.length; i++) {
			for(int j = 0; j < board[0].length; j++) {
				if(board[i][j] != 0 && !(i == lastx && j == lasty)) {
					g.setColor(Color.green);
					g.fillRect(i*cellsize+1,(board[0].length-j-1)*cellsize+1,cellsize-1,cellsize-1);
					String str = ""+board[i][j];
					int strsize = fm.stringWidth(str);
					int xmid =((int) (i*cellsize+1+0.5*cellsize-strsize*0.5));
					int ymid =((int) ((board[0].length-j-1)*cellsize+0.5*cellsize+0.5*FONTSIZE));
					g.setColor(Color.black);
					g.drawString(str,xmid,ymid);
					
				}
				if(board[i][j] != 0 && i == lastx && j == lasty) {
					g.drawImage(horse, i*cellsize+1, (board[0].length-j-1)*cellsize+1, cellsize-1, cellsize-1, this);
				}
			}
		}
	}
	
	public void setCell(int x, int y,int i) {
		this.board[x][y] = i;
		this.lastx = x;
		this.lasty = y;
	}
	
	public void drawboard(Graphics g) {
		g.setColor(Color.black);
		for(int i = 0; i <= board.length; i++) {
		g.drawLine(i*cellsize,0,i*cellsize,ysize);
		}
		for(int j = 0; j <= board[0].length; j++) {
		g.drawLine(0,j*cellsize,xsize,j*cellsize);
		}
	}
		
	public Dimension getMinimumSize() {
		return new Dimension(xsize, ysize);
	}
	
	public Dimension getPreferredSize() {
		return new Dimension(xsize, ysize);
	}
}
	
	
	