
import java.lang.*;
import java.util.Vector;

public class IK {

  // Class containing the Denavit-Hartenberg representation of the robot arm
  private static RobotJoints robotJoints;

  /* Calculate roll, pitch, and yaw for the gripper of the robot arm plus
   * the value of the gripper itself.
   */
  private static void handJointCalculation(GripperPosition pos,
                                             JointValues j) {
    j.roll = 0;
    j.pitch = -90;
    j.grip = pos.grip; 
  }

  /* Calculate the wrist coordinates from the hand coordinates.
   * If the robot's last link has some length and the tip of the robot grabs
   * the piece. At what height is the start of the robot's last link?
   */
  private static Point wristCoordinatesCalculation(GripperPosition pos) {

    Point c = new Point(pos.coords.x,pos.coords.y,pos.coords.z + robotJoints.get("roll").d + robotJoints.get("elbow").d + robotJoints.get("shoulder").d); 
    return(c);
  }

  /* Calculate the arm joints from the (x,y,z) coordinates of the wrist (the
   * last link).
   */
  private static void armJointCalculation(Point wristCoords,
              JointValues j) {
    double usingX = wristCoords.x;
    double distance = Math.sqrt(usingX * usingX + wristCoords.y * wristCoords.y);
    double armlength = robotJoints.get("elbow").a;
    double elbowangle = Math.toDegrees(Math.asin((distance/2)/armlength))*2;
    double shoulderangle = -90 + elbowangle / 2 + Math.toDegrees(Math.atan((usingX/wristCoords.y)));
    
    double answershoulder=shoulderangle, answerelbow=180-elbowangle;
    j.zed = wristCoords.z; 


	if(!(answerelbow >= -180 && answerelbow <= 151)) {
	answershoulder =  90 - elbowangle / 2 + Math.toDegrees(Math.atan((usingX/wristCoords.y)));
	answerelbow *= -1;
	}
	j.shoulder = answershoulder;
	j.elbow = answerelbow;
  }

  /* Calculate the appropriate values for all joints for position pos.
   */
  private static JointValues jointCalculation(GripperPosition pos) {
    JointValues j = new JointValues();
    Point wristCoords;

    handJointCalculation(pos,j);
    wristCoords=wristCoordinatesCalculation(pos);
    armJointCalculation(wristCoords, j);
  
    return(j);
  }

  private static void inverseKinematics(Vector p, Vector j) {

    // initialize the Denavit-Hartenberg representation
    robotJoints = new RobotJoints();
  
    for (int i =0; i < p.size(); i++) {
      GripperPosition pos = (GripperPosition) p.elementAt(i);
      j.addElement(jointCalculation(pos));
    }

  }

  public static void main(String[] args) {
    Vector p = new Vector();
    Vector j = new Vector();

    // read the gripper positions as produced by PP.java
    GripperPosition.read(p);

    // correct for errors in the arm
    // (comment-out this correction if you are solving the
    // configuration-switching problem.) 
    RobotJoints.correctCartesian(p);
          
    inverseKinematics(p, j);

    JointValues.write(j);
  }
}
